#include "Status.h"
#include "Vram.h"
#include "Stage.h"
#include "Main.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(Vram + 1, "SCORE");
    PrintS(Vram + 7, "HI-SCORE");
    PrintS(Vram + 16, "STAGE");
    PrintByteNumber2(Vram + VramRowSize + 18, CurrentStage + 1);
    PrintS(Vram+ 22, "TIME");

    {
        ptr<byte> pVram;
        pVram = Vram + 27;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            do {
                byte c;
                c = Char_Man;
                repeat (2) {
                    repeat (2) {
                        pVram = Put(pVram, c);
                        ++c;
                    }
                    pVram += VramRowSize - 2;
                }
                pVram += 2 - VramRowSize * 2;
                --i;
            } while (i > 0);
        }
    }

    PrintScore();
    PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramRowSize, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + VramRowSize + 9, HiScore);
    PrintC(pVram, '0');
}

void PrintTime()
{
    PrintByteNumber3(Vram + VramRowSize + 23, StageTime);
}

static void EraseWindow() 
{
    constexpr byte width = 12;
    constexpr byte height = 3;    
    ptr<byte> pVram;

    pVram = Vram + VramRowSize * 11 + 10;
    repeat (height) {
        repeat (width) {
            pVram = PrintC(pVram, ' ');
        }
        pVram += VramRowSize - width;
    }
}

void PrintGameOver()
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 12 + 11, "GAME OVER");
}

void PrintTimeUp() 
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 12 + 12, "TIME UP");
}

void Title()
{
    static const byte[] TitleBytes = {
		0x2c, 0x22, 0x2a, 0x1f, 
		0x2c, 0x27, 0x2b, 0x1e, 
		0x2c, 0x1d, 0x29, 0x20, 
		0x22, 0x22, 0x22, 0x1d, 
		0x1d, 0x1d, 0x1d, 0x1d, 
		0x2b, 0x2a, 0x1f, 0x2b, 
		0x2c, 0x29, 0x20, 0x2c, 
		0x21, 0x22, 0x1d, 0x21, 
		0x1d, 0x1d, 0x25, 0x1f, 
		0x2a, 0x1f, 0x2a, 0x24, 
		0x29, 0x20, 0x29, 0x20, 
		0x22, 0x1d, 0x1d, 0x22, 
		0x1d, 0x1d, 0x1d, 0x29, 
		0x25, 0x24, 0x22, 0x29, 
		0x1d, 0x22, 0x28, 0x29, 
		0x21, 0x22, 0x1e, 0x21, 
		0x20, 0x1d, 0x1d, 0x1d, 
		0x28, 0x24, 0x25, 0x24, 
		0x2c, 0x1f, 0x29, 0x24, 
		0x1e, 0x22, 0x1d, 0x22, 
		0x1d, 0x29, 0x20, 0x2c, 
		0x28, 0x29, 0x20, 0x2c, 
		0x22, 0x29, 0x20, 0x2c, 
		0x1e, 0x21, 0x1e, 0x22, 
   };

    ClearScreen(); 
    HideAllSprites();
    PrintStatus();
    {
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 7 + 4;
        p = TitleBytes;
        repeat (6) {
            repeat (4) {
                repeat (4) {
                    pVram = Put(pVram, *p);
                    ++p;
                }
                pVram += VramRowSize - 4;
            }
            pVram += 4 - VramRowSize * 4;
        }
    }
    PrintS(Vram + VramRowSize * 19 + 9 * VramStep, "PUSH SPACE KEY");
    PrintS(Vram + VramRowSize * 20 + 9 * VramStep, "OR RETURN KEY");
}